package lazabs

import java.io.{FileInputStream,InputStream,FileNotFoundException}
import parser._
import lazabs.ast._
import lazabs.ast.ASTree._
import lazabs.prover._
import lazabs.viewer._
import lazabs.cfg.MakeCFG
import lazabs.types.IntegerType
import lazabs.utils.Manip._

object Main {
  var in: InputStream = null
  var fileName = ""
  var funcName = "sc_main"
  var drawCFG = false
  var prettyPrint = false
  var log = false
  def readInputFile {
    if( fileName == "") {
      println("Input file missing")
      sys.exit(0)
    }
    try {
      in = new FileInputStream(fileName)
    } catch {
      case e: FileNotFoundException => println("No such file: " + fileName + ". Use -h for usage information" )
                       sys.exit(0)
    }
  }
  def getASTree = {
    val lexer = new Lexer(in)
    val parser = new Parser(lexer)
    val tree = parser.parse()
    if(!(tree.value.isInstanceOf[lazabs.ast.ASTree.Sobject])) {
      println("The input file is invalid")
      sys.exit(0)
    }
    val so = tree.value.asInstanceOf[lazabs.ast.ASTree.Sobject]
    val typ = lazabs.types.TypeCheck(so)
    typ
  }

  def main(args: Array[String]) {
    def arguments(args: List[String]): Unit = args match {
      case Nil =>
      case "-c" :: rest => drawCFG = true; arguments(rest)
      case "-p" :: rest => prettyPrint = true; arguments(rest)
      case mFuncName :: rest if (mFuncName.startsWith("-m:")) => funcName = "sc_" + mFuncName.drop(3); arguments(rest)
      case fn :: rest => fileName = fn;  arguments(rest)
    }
    arguments(args.toList)
    readInputFile
    val ast = getASTree
    if(prettyPrint) {println(ScalaPrinter(ast)); return}
    val cfg = MakeCFG(ast,funcName)
    if(drawCFG) {DrawGraph(cfg.transitions); return}
  }
}
