package lazabs.cfg

import lazabs.ast.ASTree._
import scala.reflect._

/**
 * The transitions in the control flow graph consists of assumes and assignments 
 */
sealed abstract class Label
case class Assume(p: Expression) extends Label
case class Assign(lhs: Expression, rhs: Expression) extends Label
case class Havoc(v: Variable) extends Label

/**
 * control flow graph
 */

case class CFGVertex(id: Int) {
  def getId = id
}
case class CFGEdge(label: Label, to: CFGVertex)

case class CFG(aStart: CFGVertex, aTransitions: Map[CFGVertex,Set[CFGEdge]]) {
  var start = aStart
  var transitions = aTransitions     // transition relation
}
