package lazabs.cfg

import lazabs.ast.ASTree._
import lazabs.types._
import lazabs.utils.Manip._

object MakeCFG {

  /**
   * gets the body of the function for constructing its CFG
   */
  def getFuncExpression(fun: String, defs: List[Declaration]): Expression = {
    def getFuncBody(defs: List[Declaration]): Expression = defs match {
      case Nil =>
          println("Error: Could not find method " + fun)
          sys.exit(0)
      case FunctionDefinition(name, _, _, body) :: ds if (fun == name) => body
      case _ :: ds => getFuncBody(ds)
    }
    getFuncBody(defs)
  }
  
  /**
   * Gets a Scala object and generates the control flow graph of the specified function
   */
  def apply(o: Sobject, fun: String): CFG = {
    val end = CFGVertex(freshVertexID) // the end state
    val funcExpression = getFuncExpression(fun,o.defs)
    val res = makeCFG(funcExpression, end)
    CFG(res._1, res._2)
  }
  
  /**
   * pointwise union of map content
   */
  def mergeMaps(l:Map[CFGVertex,Set[CFGEdge]], r:Map[CFGVertex,Set[CFGEdge]]):Map[CFGVertex,Set[CFGEdge]] = {
    var res = Map[CFGVertex,Set[CFGEdge]]().empty
    val keys = (l.keySet ++ r.keySet)
    keys.foreach( k => res += (k -> (l.getOrElse(k,Set[CFGEdge]().empty) ++ r.getOrElse(k,Set[CFGEdge]().empty))))
    res
  }

   /**
   * each vertex in the control flow graph has a unique ID
   */
  var curVertexID = -1
  def freshVertexID = {curVertexID = curVertexID + 1; curVertexID}
  
  /*
   * TO COMPLETE:
   */

  def makeCFG(e: Expression, to: CFGVertex) : (CFGVertex,Map[CFGVertex,Set[CFGEdge]]) = (to,Map())
}
