package lazabs.parser;
import java_cup.runtime.*;

%%

%class Lexer
%cupsym Symbols
%unicode
%line
%column
%public
%apiprivate

%cup

%eofval{
  return symbol(Symbols.EOF);
%eofval}

%{
    private Symbol symbol(int type) {
        return new Symbol(type, yyline, yycolumn);
    }

    private Symbol symbol(int type, Object value) {
        return new Symbol(type, yyline, yycolumn, value);
    }
    
    public int getLine() { return  yyline; }
    public int getColumn() {return yycolumn; }
%}

LineTerminator = \r|\n|\r\n
InputCharacter = [^\r\n]
WhiteSpace     = [ \t\f] | {LineTerminator}

Sign           = "+" | "-"
IntLiteral     = {Sign}? (0 | [1-9][0-9]*)
Identifier     = [a-zA-Z][a-zA-Z0-9_']* 

Comment = {TraditionalComment} | {EndOfLineComment}

TraditionalComment   = "/*" [^*] ~"*/" | "/*" "*"+ "/"
EndOfLineComment     = "//" {InputCharacter}* {LineTerminator}


%%

<YYINITIAL> {
  "&&"          { return symbol(Symbols.AND); } 
  "||"          { return symbol(Symbols.OR); } 
  "!"           { return symbol(Symbols.NOT); }
  "+"           { return symbol(Symbols.PLUS); }
  "*"           { return symbol(Symbols.TIMES); } 
  "-"           { return symbol(Symbols.MINUS); }
  "_"           { return symbol(Symbols.UNDERLINE); }
  "/"           { return symbol(Symbols.DIV); }
  "%"           { return symbol(Symbols.MOD); }
  "<="          { return symbol(Symbols.LEQ); }
  "=>"          { return symbol(Symbols.DOUBLEARROW); }  
  "<"           { return symbol(Symbols.LT); }
  ">="          { return symbol(Symbols.GEQ); }
  ">"           { return symbol(Symbols.GT); }
  "="           { return symbol(Symbols.EQ); }
  "!="          { return symbol(Symbols.NEQ); }
  "=="          { return symbol(Symbols.EQEQ); }
  "."           { return symbol(Symbols.DOT); }
  ","           { return symbol(Symbols.COMMA); }
  ":"           { return symbol(Symbols.COLON); }
  ";"           { return symbol(Symbols.SEMICOLON); }
  "("           { return symbol(Symbols.LPAREN); }
  ")"           { return symbol(Symbols.RPAREN); }
  "{"           { return symbol(Symbols.LCURLYBRACKET); }
  "}"           { return symbol(Symbols.RCURLYBRACKET); }
  "ALL"         { return symbol(Symbols.ALL); }
  "EX"          { return symbol(Symbols.EX); }
  "if"          { return symbol(Symbols.IF); }
  "else"        { return symbol(Symbols.ELSE); }
  "while"       { return symbol(Symbols.WHILE); }
  "do"          { return symbol(Symbols.DO); }
  "true"        { return symbol(Symbols.TRUE); }
  "false"       { return symbol(Symbols.FALSE); }
  "null"        { return symbol(Symbols.NULL); }
  "Int"         { return symbol(Symbols.INT); }
  "Boolean"     { return symbol(Symbols.BOOLEAN); }  
  "String"      { return symbol(Symbols.STRING); }
  "Unit"        { return symbol(Symbols.UNIT); }    
  "new"         { return symbol(Symbols.NEW); }
  "var"         { return symbol(Symbols.VAR); }
  "val"         { return symbol(Symbols.VAL); }
  "def"         { return symbol(Symbols.DEF); }
  "import"      { return symbol(Symbols.IMPORT); }
  "object"		{ return symbol(Symbols.OBJECT); }
  "until"	    { return symbol(Symbols.UNTIL); }
  {IntLiteral}  { return symbol(Symbols.NUMBER, new Integer(yytext())); }
  {Identifier}  { return symbol(Symbols.ID, new String(yytext())); } 
  {WhiteSpace}  { /* skip */ }
  {Comment}		{ /* skip */ }
  
}


. | \n          { System.err.println("Illegal character '" + yytext() + "'" + " Scala Lexial Analyzer in " + yyline + " " + yycolumn); }
